@@ -55,13 +55,16 @@ class TestAsyncClient(unittest.TestCase):
             c.connect(
                 'url',
                 headers='headers',
+                auth='auth',
                 transports='transports',
                 namespaces=['/foo', '/', '/bar'],
                 socketio_path='path',
+                wait=False,
             )
         )
         assert c.connection_url == 'url'
         assert c.connection_headers == 'headers'
+        assert c.connection_auth == 'auth'
         assert c.connection_transports == 'transports'
         assert c.connection_namespaces == ['/foo', '/', '/bar']
         assert c.socketio_path == 'path'
@@ -72,6 +75,30 @@ class TestAsyncClient(unittest.TestCase):
             engineio_path='path',
         )
 
+    def test_connect_functions(self):
+        async def headers():
+            return 'headers'
+
+        c = asyncio_client.AsyncClient()
+        c.eio.connect = AsyncMock()
+        _run(
+            c.connect(
+                lambda: 'url',
+                headers=headers,
+                auth='auth',
+                transports='transports',
+                namespaces=['/foo', '/', '/bar'],
+                socketio_path='path',
+                wait=False,
+            )
+        )
+        c.eio.connect.mock.assert_called_once_with(
+            'url',
+            headers='headers',
+            transports='transports',
+            engineio_path='path',
+        )
+
     def test_connect_one_namespace(self):
         c = asyncio_client.AsyncClient()
         c.eio.connect = AsyncMock()
@@ -82,6 +109,7 @@ class TestAsyncClient(unittest.TestCase):
                 transports='transports',
                 namespaces='/foo',
                 socketio_path='path',
+                wait=False,
             )
         )
         assert c.connection_url == 'url'
@@ -107,6 +135,7 @@ class TestAsyncClient(unittest.TestCase):
                 headers='headers',
                 transports='transports',
                 socketio_path='path',
+                wait=False,
             )
         )
         assert c.connection_url == 'url'
@@ -131,6 +160,7 @@ class TestAsyncClient(unittest.TestCase):
                 headers='headers',
                 transports='transports',
                 socketio_path='path',
+                wait=False,
             )
         )
         assert c.connection_url == 'url'
@@ -159,9 +189,86 @@ class TestAsyncClient(unittest.TestCase):
                     headers='headers',
                     transports='transports',
                     socketio_path='path',
+                    wait=False,
                 )
             )
 
+    def test_connect_twice(self):
+        c = asyncio_client.AsyncClient()
+        c.eio.connect = AsyncMock()
+        _run(
+            c.connect(
+                'url',
+                wait=False,
+            )
+        )
+        with pytest.raises(exceptions.ConnectionError):
+            _run(
+                c.connect(
+                    'url',
+                    wait=False,
+                )
+            )
+
+    def test_connect_wait_single_namespace(self):
+        c = asyncio_client.AsyncClient()
+        c.eio.connect = AsyncMock()
+        c._connect_event = mock.MagicMock()
+
+        async def mock_connect():
+            c.namespaces = {'/': '123'}
+            return True
+
+        c._connect_event.wait = mock_connect
+        _run(
+            c.connect(
+                'url',
+                wait=True,
+                wait_timeout=0.01,
+            )
+        )
+        assert c.connected is True
+
+    def test_connect_wait_two_namespaces(self):
+        c = asyncio_client.AsyncClient()
+        c.eio.connect = AsyncMock()
+        c._connect_event = mock.MagicMock()
+
+        async def mock_connect():
+            if c.namespaces == {}:
+                c.namespaces = {'/bar': '123'}
+                return True
+            elif c.namespaces == {'/bar': '123'}:
+                c.namespaces = {'/bar': '123', '/foo': '456'}
+                return True
+            return False
+
+        c._connect_event.wait = mock_connect
+        _run(
+            c.connect(
+                'url',
+                namespaces=['/foo', '/bar'],
+                wait=True,
+                wait_timeout=0.01,
+            )
+        )
+        assert c.connected is True
+        assert c.namespaces == {'/bar': '123', '/foo': '456'}
+
+    def test_connect_timeout(self):
+        c = asyncio_client.AsyncClient()
+        c.eio.connect = AsyncMock()
+        c.disconnect = AsyncMock()
+        with pytest.raises(exceptions.ConnectionError):
+            _run(
+                c.connect(
+                    'url',
+                    wait=True,
+                    wait_timeout=0.01,
+                )
+            )
+        c.disconnect.mock.assert_called_once_with()
+
     def test_wait_no_reconnect(self):
         c = asyncio_client.AsyncClient()
         c.eio.wait = AsyncMock()
@@ -406,6 +513,7 @@ class TestAsyncClient(unittest.TestCase):
 
     def test_disconnect(self):
         c = asyncio_client.AsyncClient()
+        c.connected = True
         c.namespaces = {'/': '1'}
         c._trigger_event = AsyncMock()
         c._send_packet = AsyncMock()
@@ -413,6 +521,7 @@ class TestAsyncClient(unittest.TestCase):
         c.eio.disconnect = AsyncMock()
         c.eio.state = 'connected'
         _run(c.disconnect())
+        assert c.connected
         assert c._trigger_event.mock.call_count == 0
         assert c._send_packet.mock.call_count == 1
         expected_packet = packet.Packet(packet.DISCONNECT, namespace='/')
@@ -484,29 +593,35 @@ class TestAsyncClient(unittest.TestCase):
 
     def test_handle_connect(self):
         c = asyncio_client.AsyncClient()
+        c._connect_event = mock.MagicMock()
         c._trigger_event = AsyncMock()
         c._send_packet = AsyncMock()
         _run(c._handle_connect('/', {'sid': '123'}))
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.mock.assert_called_once_with('connect', namespace='/')
         c._send_packet.mock.assert_not_called()
 
     def test_handle_connect_with_namespaces(self):
         c = asyncio_client.AsyncClient()
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = AsyncMock()
         c._send_packet = AsyncMock()
         _run(c._handle_connect('/', {'sid': '3'}))
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.mock.assert_called_once_with('connect', namespace='/')
         assert c.namespaces == {'/': '3', '/foo': '1', '/bar': '2'}
 
     def test_handle_connect_namespace(self):
         c = asyncio_client.AsyncClient()
         c.namespaces = {'/foo': '1'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = AsyncMock()
         c._send_packet = AsyncMock()
         _run(c._handle_connect('/foo', {'sid': '123'}))
         _run(c._handle_connect('/bar', {'sid': '2'}))
         assert c._trigger_event.mock.call_count == 1
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.mock.assert_called_once_with(
             'connect', namespace='/bar')
         assert c.namespaces == {'/foo': '1', '/bar': '2'}
@@ -656,11 +771,13 @@ class TestAsyncClient(unittest.TestCase):
     def test_handle_error(self):
         c = asyncio_client.AsyncClient()
         c.connected = True
+        c._connect_event = mock.MagicMock()
         c._trigger_event = AsyncMock()
         c.namespaces = {'/foo': '1', '/bar': '2'}
         _run(c._handle_error('/', 'error'))
         assert c.namespaces == {}
         assert not c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.mock.assert_called_once_with(
             'connect_error', '/', 'error'
         )
@@ -668,21 +785,25 @@ class TestAsyncClient(unittest.TestCase):
     def test_handle_error_with_no_arguments(self):
         c = asyncio_client.AsyncClient()
         c.connected = True
+        c._connect_event = mock.MagicMock()
         c._trigger_event = AsyncMock()
         c.namespaces = {'/foo': '1', '/bar': '2'}
         _run(c._handle_error('/', None))
         assert c.namespaces == {}
         assert not c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.mock.assert_called_once_with('connect_error', '/')
 
     def test_handle_error_namespace(self):
         c = asyncio_client.AsyncClient()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = AsyncMock()
         _run(c._handle_error('/bar', ['error', 'message']))
         assert c.namespaces == {'/foo': '1'}
         assert c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.mock.assert_called_once_with(
             'connect_error', '/bar', 'error', 'message'
         )
@@ -691,33 +812,46 @@ class TestAsyncClient(unittest.TestCase):
         c = asyncio_client.AsyncClient()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = AsyncMock()
         _run(c._handle_error('/bar', None))
         assert c.namespaces == {'/foo': '1'}
         assert c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.mock.assert_called_once_with('connect_error', '/bar')
 
     def test_handle_error_unknown_namespace(self):
         c = asyncio_client.AsyncClient()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         _run(c._handle_error('/baz', 'error'))
         assert c.namespaces == {'/foo': '1', '/bar': '2'}
         assert c.connected
+        c._connect_event.set.assert_called_once_with()
 
     def test_trigger_event(self):
         c = asyncio_client.AsyncClient()
         handler = mock.MagicMock()
+        catchall_handler = mock.MagicMock()
         c.on('foo', handler)
+        c.on('*', catchall_handler)
         _run(c._trigger_event('foo', '/', 1, '2'))
+        _run(c._trigger_event('bar', '/', 1, '2', 3))
+        _run(c._trigger_event('connect', '/'))  # should not trigger
         handler.assert_called_once_with(1, '2')
+        catchall_handler.assert_called_once_with('bar', 1, '2', 3)
 
     def test_trigger_event_namespace(self):
         c = asyncio_client.AsyncClient()
         handler = AsyncMock()
+        catchall_handler = AsyncMock()
         c.on('foo', handler, namespace='/bar')
+        c.on('*', catchall_handler, namespace='/bar')
         _run(c._trigger_event('foo', '/bar', 1, '2'))
+        _run(c._trigger_event('bar', '/bar', 1, '2', 3))
         handler.mock.assert_called_once_with(1, '2')
+        catchall_handler.mock.assert_called_once_with('bar', 1, '2', 3)
 
     def test_trigger_event_class_namespace(self):
         c = asyncio_client.AsyncClient()
@@ -758,8 +892,6 @@ class TestAsyncClient(unittest.TestCase):
             side_effect=[ValueError, exceptions.ConnectionError, None]
         )
         _run(c._handle_reconnect())
-        print(wait_for.mock.call_count)  # logging to debug #572
-        print(wait_for.mock.call_args_list)
         assert wait_for.mock.call_count == 3
         assert [x[0][1] for x in asyncio.wait_for.mock.call_args_list] == [
             1.5,
@@ -781,8 +913,6 @@ class TestAsyncClient(unittest.TestCase):
             side_effect=[ValueError, exceptions.ConnectionError, None]
         )
         _run(c._handle_reconnect())
-        print(wait_for.mock.call_count)  # logging to debug #572
-        print(wait_for.mock.call_args_list)
         assert wait_for.mock.call_count == 3
         assert [x[0][1] for x in asyncio.wait_for.mock.call_args_list] == [
             1.5,
@@ -798,14 +928,12 @@ class TestAsyncClient(unittest.TestCase):
     )
     @mock.patch('socketio.client.random.random', side_effect=[1, 0, 0.5])
     def test_handle_reconnect_max_attempts(self, random, wait_for):
-        c = asyncio_client.AsyncClient(reconnection_attempts=2)
+        c = asyncio_client.AsyncClient(reconnection_attempts=2, logger=True)
         c._reconnect_task = 'foo'
         c.connect = AsyncMock(
             side_effect=[ValueError, exceptions.ConnectionError, None]
         )
         _run(c._handle_reconnect())
-        print(wait_for.mock.call_count)  # logging to debug #572
-        print(wait_for.mock.call_args_list)
         assert wait_for.mock.call_count == 2
         assert [x[0][1] for x in asyncio.wait_for.mock.call_args_list] == [
             1.5,
@@ -820,14 +948,12 @@ class TestAsyncClient(unittest.TestCase):
     )
     @mock.patch('socketio.client.random.random', side_effect=[1, 0, 0.5])
     def test_handle_reconnect_aborted(self, random, wait_for):
-        c = asyncio_client.AsyncClient()
+        c = asyncio_client.AsyncClient(logger=True)
         c._reconnect_task = 'foo'
         c.connect = AsyncMock(
             side_effect=[ValueError, exceptions.ConnectionError, None]
         )
         _run(c._handle_reconnect())
-        print(wait_for.mock.call_count)  # logging to debug #572
-        print(wait_for.mock.call_args_list)
         assert wait_for.mock.call_count == 2
         assert [x[0][1] for x in asyncio.wait_for.mock.call_args_list] == [
             1.5,
@@ -835,21 +961,47 @@ class TestAsyncClient(unittest.TestCase):
         ]
         assert c._reconnect_task == 'foo'
 
-    def test_eio_connect(self):
+    def test_handle_eio_connect(self):
         c = asyncio_client.AsyncClient()
         c.connection_namespaces = ['/', '/foo']
+        c.connection_auth = 'auth'
         c._send_packet = AsyncMock()
         c.eio.sid = 'foo'
         assert c.sid is None
         _run(c._handle_eio_connect())
         assert c.sid == 'foo'
         assert c._send_packet.mock.call_count == 2
-        expected_packet = packet.Packet(packet.CONNECT, namespace='/')
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/')
         assert (
             c._send_packet.mock.call_args_list[0][0][0].encode()
             == expected_packet.encode()
         )
-        expected_packet = packet.Packet(packet.CONNECT, namespace='/foo')
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/foo')
+        assert (
+            c._send_packet.mock.call_args_list[1][0][0].encode()
+            == expected_packet.encode()
+        )
+
+    def test_handle_eio_connect_function(self):
+        c = asyncio_client.AsyncClient()
+        c.connection_namespaces = ['/', '/foo']
+        c.connection_auth = lambda: 'auth'
+        c._send_packet = AsyncMock()
+        c.eio.sid = 'foo'
+        assert c.sid is None
+        _run(c._handle_eio_connect())
+        assert c.sid == 'foo'
+        assert c._send_packet.mock.call_count == 2
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/')
+        assert (
+            c._send_packet.mock.call_args_list[0][0][0].encode()
+            == expected_packet.encode()
+        )
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/foo')
         assert (
             c._send_packet.mock.call_args_list[1][0][0].encode()
             == expected_packet.encode()
@@ -905,7 +1057,7 @@ class TestAsyncClient(unittest.TestCase):
             _run(c._handle_eio_message('9'))
 
     def test_eio_disconnect(self):
-        c = asyncio_client.AsyncClient()
+        c = asyncio_client.AsyncClient(reconnection=False)
         c.namespaces = {'/': '1'}
         c.connected = True
         c._trigger_event = AsyncMock()
@@ -919,7 +1071,7 @@ class TestAsyncClient(unittest.TestCase):
         assert not c.connected
 
     def test_eio_disconnect_namespaces(self):
-        c = asyncio_client.AsyncClient()
+        c = asyncio_client.AsyncClient(reconnection=False)
         c.namespaces = {'/foo': '1', '/bar': '2'}
         c.connected = True
         c._trigger_event = AsyncMock()
