@@ -1,16 +1,17 @@
-import json
 import logging
 import sys
 import unittest
 from unittest import mock
 
 from engineio import exceptions as engineio_exceptions
+from engineio import json
 from engineio import packet as engineio_packet
 import pytest
 
 from socketio import asyncio_namespace
 from socketio import client
 from socketio import exceptions
+from socketio import msgpack_packet
 from socketio import namespace
 from socketio import packet
 
@@ -28,6 +29,7 @@ class TestClient(unittest.TestCase):
             reconnection_delay=5,
             reconnection_delay_max=10,
             randomization_factor=0.2,
+            handle_sigint=False,
             foo='bar',
         )
         assert not c.reconnection
@@ -35,7 +37,9 @@ class TestClient(unittest.TestCase):
         assert c.reconnection_delay == 5
         assert c.reconnection_delay_max == 10
         assert c.randomization_factor == 0.2
-        engineio_client_class().assert_called_once_with(foo='bar')
+        assert not c.handle_sigint
+        engineio_client_class().assert_called_once_with(
+            foo='bar', handle_sigint=False)
         assert c.connection_url is None
         assert c.connection_headers is None
         assert c.connection_transports is None
@@ -49,8 +53,20 @@ class TestClient(unittest.TestCase):
         assert c.callbacks == {}
         assert c._binary_packet is None
         assert c._reconnect_task is None
+        assert c.packet_class == packet.Packet
 
-    def test_custon_json(self):
+    def test_msgpack(self):
+        c = client.Client(serializer='msgpack')
+        assert c.packet_class == msgpack_packet.MsgPackPacket
+
+    def test_custom_serializer(self):
+        class CustomPacket(packet.Packet):
+            pass
+
+        c = client.Client(serializer=CustomPacket)
+        assert c.packet_class == CustomPacket
+
+    def test_custom_json(self):
         client.Client()
         assert packet.Packet.json == json
         assert engineio_packet.Packet.json == json
@@ -76,7 +92,8 @@ class TestClient(unittest.TestCase):
     @mock.patch('socketio.client.Client._engineio_client_class')
     def test_engineio_logger(self, engineio_client_class):
         client.Client(engineio_logger='foo')
-        engineio_client_class().assert_called_once_with(logger='foo')
+        engineio_client_class().assert_called_once_with(
+            handle_sigint=True, logger='foo')
 
     def test_on_event(self):
         c = client.Client()
@@ -154,12 +171,15 @@ class TestClient(unittest.TestCase):
         c.connect(
             'url',
             headers='headers',
+            auth='auth',
             transports='transports',
             namespaces=['/foo', '/', '/bar'],
             socketio_path='path',
+            wait=False,
         )
         assert c.connection_url == 'url'
         assert c.connection_headers == 'headers'
+        assert c.connection_auth == 'auth'
         assert c.connection_transports == 'transports'
         assert c.connection_namespaces == ['/foo', '/', '/bar']
         assert c.socketio_path == 'path'
@@ -170,6 +190,25 @@ class TestClient(unittest.TestCase):
             engineio_path='path',
         )
 
+    def test_connect_functions(self):
+        c = client.Client()
+        c.eio.connect = mock.MagicMock()
+        c.connect(
+            lambda: 'url',
+            headers=lambda: 'headers',
+            auth='auth',
+            transports='transports',
+            namespaces=['/foo', '/', '/bar'],
+            socketio_path='path',
+            wait=False,
+        )
+        c.eio.connect.assert_called_once_with(
+            'url',
+            headers='headers',
+            transports='transports',
+            engineio_path='path',
+        )
+
     def test_connect_one_namespace(self):
         c = client.Client()
         c.eio.connect = mock.MagicMock()
@@ -179,6 +218,7 @@ class TestClient(unittest.TestCase):
             transports='transports',
             namespaces='/foo',
             socketio_path='path',
+            wait=False,
         )
         assert c.connection_url == 'url'
         assert c.connection_headers == 'headers'
@@ -202,6 +242,7 @@ class TestClient(unittest.TestCase):
             headers='headers',
             transports='transports',
             socketio_path='path',
+            wait=False,
         )
         assert c.connection_url == 'url'
         assert c.connection_headers == 'headers'
@@ -224,6 +265,7 @@ class TestClient(unittest.TestCase):
             headers='headers',
             transports='transports',
             socketio_path='path',
+            wait=False,
         )
         assert c.connection_url == 'url'
         assert c.connection_headers == 'headers'
@@ -250,8 +292,77 @@ class TestClient(unittest.TestCase):
                 headers='headers',
                 transports='transports',
                 socketio_path='path',
+                wait=False,
             )
 
+    def test_connect_twice(self):
+        c = client.Client()
+        c.eio.connect = mock.MagicMock()
+        c.connect(
+            'url',
+            wait=False,
+        )
+        with pytest.raises(exceptions.ConnectionError):
+            c.connect(
+                'url',
+                wait=False,
+            )
+
+    def test_connect_wait_single_namespace(self):
+        c = client.Client()
+        c.eio.connect = mock.MagicMock()
+        c._connect_event = mock.MagicMock()
+
+        def mock_connect(timeout):
+            assert timeout == 0.01
+            c.namespaces = {'/': '123'}
+            return True
+
+        c._connect_event.wait = mock_connect
+        c.connect(
+            'url',
+            wait=True,
+            wait_timeout=0.01,
+        )
+        assert c.connected is True
+
+    def test_connect_wait_two_namespaces(self):
+        c = client.Client()
+        c.eio.connect = mock.MagicMock()
+        c._connect_event = mock.MagicMock()
+
+        def mock_connect(timeout):
+            assert timeout == 0.01
+            if c.namespaces == {}:
+                c.namespaces = {'/bar': '123'}
+                return True
+            elif c.namespaces == {'/bar': '123'}:
+                c.namespaces = {'/bar': '123', '/foo': '456'}
+                return True
+            return False
+
+        c._connect_event.wait = mock_connect
+        c.connect(
+            'url',
+            namespaces=['/foo', '/bar'],
+            wait=True,
+            wait_timeout=0.01,
+        )
+        assert c.connected is True
+        assert c.namespaces == {'/bar': '123', '/foo': '456'}
+
+    def test_connect_timeout(self):
+        c = client.Client()
+        c.eio.connect = mock.MagicMock()
+        c.disconnect = mock.MagicMock()
+        with pytest.raises(exceptions.ConnectionError):
+            c.connect(
+                'url',
+                wait=True,
+                wait_timeout=0.01,
+            )
+        c.disconnect.assert_called_once_with()
+
     def test_wait_no_reconnect(self):
         c = client.Client()
         c.eio.wait = mock.MagicMock()
@@ -508,12 +619,14 @@ class TestClient(unittest.TestCase):
 
     def test_disconnect(self):
         c = client.Client()
+        c.connected = True
         c.namespaces = {'/': '1'}
         c._trigger_event = mock.MagicMock()
         c._send_packet = mock.MagicMock()
         c.eio = mock.MagicMock()
         c.eio.state = 'connected'
         c.disconnect()
+        assert c.connected
         assert c._trigger_event.call_count == 0
         assert c._send_packet.call_count == 1
         expected_packet = packet.Packet(packet.DISCONNECT, namespace='/')
@@ -600,30 +713,36 @@ class TestClient(unittest.TestCase):
 
     def test_handle_connect(self):
         c = client.Client()
+        c._connect_event = mock.MagicMock()
         c._trigger_event = mock.MagicMock()
         c._send_packet = mock.MagicMock()
         c._handle_connect('/', {'sid': '123'})
         assert c.namespaces == {'/': '123'}
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.assert_called_once_with('connect', namespace='/')
         c._send_packet.assert_not_called()
 
     def test_handle_connect_with_namespaces(self):
         c = client.Client()
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = mock.MagicMock()
         c._send_packet = mock.MagicMock()
         c._handle_connect('/', {'sid': '3'})
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.assert_called_once_with('connect', namespace='/')
         assert c.namespaces == {'/': '3', '/foo': '1', '/bar': '2'}
 
     def test_handle_connect_namespace(self):
         c = client.Client()
         c.namespaces = {'/foo': '1'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = mock.MagicMock()
         c._send_packet = mock.MagicMock()
         c._handle_connect('/foo', {'sid': '123'})
         c._handle_connect('/bar', {'sid': '2'})
         assert c._trigger_event.call_count == 1
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.assert_called_once_with('connect', namespace='/bar')
         assert c.namespaces == {'/foo': '1', '/bar': '2'}
 
@@ -760,30 +879,36 @@ class TestClient(unittest.TestCase):
         c = client.Client()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = mock.MagicMock()
         c._handle_error('/', 'error')
         assert c.namespaces == {}
         assert not c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.assert_called_once_with('connect_error', '/', 'error')
 
     def test_handle_error_with_no_arguments(self):
         c = client.Client()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = mock.MagicMock()
         c._handle_error('/', None)
         assert c.namespaces == {}
         assert not c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.assert_called_once_with('connect_error', '/')
 
     def test_handle_error_namespace(self):
         c = client.Client()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = mock.MagicMock()
         c._handle_error('/bar', ['error', 'message'])
         assert c.namespaces == {'/foo': '1'}
         assert c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.assert_called_once_with(
             'connect_error', '/bar', 'error', 'message'
         )
@@ -792,33 +917,46 @@ class TestClient(unittest.TestCase):
         c = client.Client()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._trigger_event = mock.MagicMock()
         c._handle_error('/bar', None)
         assert c.namespaces == {'/foo': '1'}
         assert c.connected
+        c._connect_event.set.assert_called_once_with()
         c._trigger_event.assert_called_once_with('connect_error', '/bar')
 
     def test_handle_error_unknown_namespace(self):
         c = client.Client()
         c.connected = True
         c.namespaces = {'/foo': '1', '/bar': '2'}
+        c._connect_event = mock.MagicMock()
         c._handle_error('/baz', 'error')
         assert c.namespaces == {'/foo': '1', '/bar': '2'}
         assert c.connected
+        c._connect_event.set.assert_called_once_with()
 
     def test_trigger_event(self):
         c = client.Client()
         handler = mock.MagicMock()
+        catchall_handler = mock.MagicMock()
         c.on('foo', handler)
+        c.on('*', catchall_handler)
         c._trigger_event('foo', '/', 1, '2')
+        c._trigger_event('bar', '/', 1, '2', 3)
+        c._trigger_event('connect', '/')  # should not trigger
         handler.assert_called_once_with(1, '2')
+        catchall_handler.assert_called_once_with('bar', 1, '2', 3)
 
     def test_trigger_event_namespace(self):
         c = client.Client()
         handler = mock.MagicMock()
+        catchall_handler = mock.MagicMock()
         c.on('foo', handler, namespace='/bar')
+        c.on('*', catchall_handler, namespace='/bar')
         c._trigger_event('foo', '/bar', 1, '2')
+        c._trigger_event('bar', '/bar', 1, '2', 3)
         handler.assert_called_once_with(1, '2')
+        catchall_handler.assert_called_once_with('bar', 1, '2', 3)
 
     def test_trigger_event_class_namespace(self):
         c = client.Client()
@@ -850,6 +988,7 @@ class TestClient(unittest.TestCase):
     def test_handle_reconnect(self, random):
         c = client.Client()
         c._reconnect_task = 'foo'
+        c._reconnect_abort = c.eio.create_event()
         c._reconnect_abort.wait = mock.MagicMock(return_value=False)
         c.connect = mock.MagicMock(
             side_effect=[ValueError, exceptions.ConnectionError, None]
@@ -867,6 +1006,7 @@ class TestClient(unittest.TestCase):
     def test_handle_reconnect_max_delay(self, random):
         c = client.Client(reconnection_delay_max=3)
         c._reconnect_task = 'foo'
+        c._reconnect_abort = c.eio.create_event()
         c._reconnect_abort.wait = mock.MagicMock(return_value=False)
         c.connect = mock.MagicMock(
             side_effect=[ValueError, exceptions.ConnectionError, None]
@@ -884,6 +1024,7 @@ class TestClient(unittest.TestCase):
     def test_handle_reconnect_max_attempts(self, random):
         c = client.Client(reconnection_attempts=2)
         c._reconnect_task = 'foo'
+        c._reconnect_abort = c.eio.create_event()
         c._reconnect_abort.wait = mock.MagicMock(return_value=False)
         c.connect = mock.MagicMock(
             side_effect=[ValueError, exceptions.ConnectionError, None]
@@ -900,6 +1041,7 @@ class TestClient(unittest.TestCase):
     def test_handle_reconnect_aborted(self, random):
         c = client.Client()
         c._reconnect_task = 'foo'
+        c._reconnect_abort = c.eio.create_event()
         c._reconnect_abort.wait = mock.MagicMock(side_effect=[False, True])
         c.connect = mock.MagicMock(side_effect=exceptions.ConnectionError)
         c._handle_reconnect()
@@ -913,18 +1055,44 @@ class TestClient(unittest.TestCase):
     def test_handle_eio_connect(self):
         c = client.Client()
         c.connection_namespaces = ['/', '/foo']
+        c.connection_auth = 'auth'
         c._send_packet = mock.MagicMock()
         c.eio.sid = 'foo'
         assert c.sid is None
         c._handle_eio_connect()
         assert c.sid == 'foo'
         assert c._send_packet.call_count == 2
-        expected_packet = packet.Packet(packet.CONNECT, namespace='/')
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/')
         assert (
             c._send_packet.call_args_list[0][0][0].encode()
             == expected_packet.encode()
         )
-        expected_packet = packet.Packet(packet.CONNECT, namespace='/foo')
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/foo')
+        assert (
+            c._send_packet.call_args_list[1][0][0].encode()
+            == expected_packet.encode()
+        )
+
+    def test_handle_eio_connect_function(self):
+        c = client.Client()
+        c.connection_namespaces = ['/', '/foo']
+        c.connection_auth = lambda: 'auth'
+        c._send_packet = mock.MagicMock()
+        c.eio.sid = 'foo'
+        assert c.sid is None
+        c._handle_eio_connect()
+        assert c.sid == 'foo'
+        assert c._send_packet.call_count == 2
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/')
+        assert (
+            c._send_packet.call_args_list[0][0][0].encode()
+            == expected_packet.encode()
+        )
+        expected_packet = packet.Packet(
+            packet.CONNECT, data='auth', namespace='/foo')
         assert (
             c._send_packet.call_args_list[1][0][0].encode()
             == expected_packet.encode()
